#pragma once

#include "blockforest/StructuredBlockForest.h"

#include <memory>

#include "core/cell/all.h"

namespace walberla::experimental::sweep
{

class SerialSweeper
{
 private:
   std::shared_ptr< StructuredBlockForest > blocks_;

 public:
   SerialSweeper(const std::shared_ptr< StructuredBlockForest >& blocks) : blocks_{ blocks } {}

   void sweep(std::function< void(IBlock&) > func)
   {
      for (auto& block : *blocks_)
         func(block);
   }

   void sweep(std::function< void(IBlock*) > func)
   {
      for (auto& block : *blocks_)
         func(&block);
   }

   void forAllBlocks(std::function< void(IBlock&) > func) { sweep(func); }

   void forAllBlocks(std::function< void(IBlock*) > func) { sweep(func); }

   void forAllCells(std::function< void(Cell) > func)
   {
      for (uint_t z = 0; z < blocks_->getNumberOfZCellsPerBlock(); ++z)
         for (uint_t y = 0; y < blocks_->getNumberOfYCellsPerBlock(); ++y)
            for (uint_t x = 0; x < blocks_->getNumberOfXCellsPerBlock(); ++x)
               func({ x, y, z });
   }

   void forAllCells(const CellInterval& ci, std::function< void(Cell) > func)
   {
      for (uint_t z = ci.zMin(); z <= ci.zMax(); ++z)
         for (uint_t y = ci.yMin(); y <= ci.yMax(); ++y)
            for (uint_t x = ci.xMin(); x <= ci.xMax(); ++x)
               func({ x, y, z });
   }
};

} // namespace walberla::experimental::sweep
