import nox

nox.options.sessions = ["lint", "typecheck"]


def editable_install(session: nox.Session):
    session.install("-r", "cmake/codegen-requirements.txt")
    session.install("-e", ".")


@nox.session(python="3.10", tags=["qa", "code-quality"])
def lint(session: nox.Session):
    """Lint code using flake8"""

    session.install("flake8")
    session.run("flake8", "src/walberla/codegen")


@nox.session(python="3.10", tags=["qa", "code-quality"])
def typecheck(session: nox.Session):
    """Run MyPy for static type checking"""
    editable_install(session)
    session.install("mypy")
    session.run("mypy", "src/walberla/codegen")


@nox.session
def user_manual(session: nox.Session):
    session.chdir("user_manual")
    session.install("-r", "requirements.txt")

    if "--clean" in session.posargs:
        session.run("make", "clean", external=True)

    session.run("make", "html", external=True)
