(EnvSetup)=
# CMake Project Setup

This chapter describes the necessary steps to set up a CMake project to use waLBerla
with the nextcodegen features provided by sfg-walberla.

## Prequesites

You are going to need at least
- a C++ compiler supporting at least C++17;
- a Python installation of version >= 3.10;
- a CMake installation of version >= 3.24;
- an up-to-date installation of Git.

Also, the following optional dependencies are highly recommended:
- clang-format for prettier code generator output

If you are starting from scratch, create a new, empty directory for your CMake project.
Then, set up its `CMakeLists.txt` with the following minimal settings:

:::{literalinclude} CMakeLists.txt
:lines: 1-2
:::

On the other hand. if you already have a CMake project set up, you can just as well extend it.

## Add sfg-walberla to your CMake Project

There are many ways to include `waLBerla` and `sfg-walberla` into your project:
you can clone them locally and use `add_subdirectory`, add them as submodules, et cetera.
For getting started, though, the easiest way is to use [FetchContent][FetchContent] to dynamically
pull their sources into your build tree.

Place the following code in your `CMakeLists.txt`:

::::{card} {download}`CMakeLists.txt`
:::{literalinclude} CMakeLists.txt
:lines: 4-
:::
::::

Now, you can run `cmake` to configure your build system:

```bash
mkdir build
cmake -S . -B build
```

If, near the end of the long configuration log, you see two messages like this:

```
-- Found pystencils Source File Generator (Version 0.1a4+25.g9d3e553)
-- Using Python interpreter <your-project-dir>/build/_deps/sfg-walberla-build/codegen-venv/bin/python for SFG generator scripts.
```

Then your build system setup was successful!
Now you can get down to business and populate your project with simulation applications.
