import sympy as sp
from pystencilssfg import SourceFileGenerator

from pystencils import fields, Field, Target, CreateKernelConfig
from lbmpy import LBStencil, Stencil, LBMConfig, LBMOptimisation, create_lb_update_rule
from lbmpy.macroscopic_value_kernels import macroscopic_values_setter


from walberla.codegen import Sweep, get_build_config

with SourceFileGenerator() as sfg:
    sfg.namespace("FreeSlipBubble::gen")

    stencil = LBStencil(Stencil.D3Q19)
    d, q = stencil.D, stencil.Q
    pdfs: Field
    pdfs_tmp: Field
    pdfs, pdfs_tmp, rho, u = fields(
        f"f({q}), f_tmp({q}), rho, u({d}): double[{d}D]", layout="fzyx"
    )  # type: ignore

    lbm_config = LBMConfig(
        stencil=Stencil.D3Q19,
        output={"density": rho, "velocity": u},
    )
    lbm_opt = LBMOptimisation(
        symbolic_field=pdfs,
        symbolic_temporary_field=pdfs_tmp,
    )

    wlb_config = get_build_config(sfg)
    wlb_config.override.target = Target.GenericCPU

    lb_update = create_lb_update_rule(lbm_config=lbm_config, lbm_optimisation=lbm_opt)
    lb_method = lb_update.method
    assert lb_update is not None

    lb_update_sweep = Sweep("LbStreamCollide", lb_update)
    lb_update_sweep.swap_fields(pdfs, pdfs_tmp)
    sfg.generate(lb_update_sweep)

    lb_init = macroscopic_values_setter(
        lb_update.method,
        density=sp.Symbol("density"),
        velocity=sp.symbols(f"velocity_:{d}"),
        pdfs=pdfs,
    )
    lb_init_sweep = Sweep("LbInit", lb_init)
    sfg.generate(lb_init_sweep)

    #   begin irregular-freeslip
    from walberla.codegen.boundaries import FreeSlip

    freeslip = FreeSlip(
        "FreeSlip",
        lb_method,
        pdfs,
        wall_orientation=FreeSlip.IRREGULAR,
    )

    sfg.generate(freeslip)
    #   end irregular-freeslip
