(guide-boundary-conditions)=
# Boundary Conditions

## Sparse Boundary Sweeps for Irregular Geometries

To cover the boundary conditions at surfaces with irregular geometries
(i.e. surfaces not aligned with the cell grid),
we need to use a sparse implementation of the boundary sweep.

### Generating Sparse Link-Wise LBM Boundary Sweeps

For link-wise lattice Boltzmann boundary conditions, sparsity is enabled by
setting the `wall_normal` parameter to `IRREGULAR`:

```{literalinclude} ../examples/FreeSlipBubble/LbmAlgorithms.py
:language: Python
:start-after: "#   begin irregular-freeslip"
:end-before: "#   end irregular-freeslip"
:linenos:
:dedent: 4
:emphasize-lines: 7
```

The above code will generate two classes:
 - The `FreeSlip` sweep class, which is a [sparse sweep](#guide-sparse-sweeps) that applies
   the free-slip boundary condition at each lattice link crossing the boundary surface;
 - and the `FreeSlipFactory` class; which produces instances of the `FreeSlip` sweep and populates its
   index vector from user-provided geometry information.

### Populating the Index Vector

Sparse boundary sweeps are created, and their index vectors populated, by their associated factory.
Create the factory object by passing it any required parameters:

```{literalinclude} ../examples/FreeSlipBubble/FreeSlipBubble.cpp
:language: C++
:start-after: "// begin freeSlipFactory"
:end-before: "// end freeSlipFactory"
:dedent: 8
```

#### From a Flag Field

One possibility to populate the sparse sweep is to extract boundary information stored in a *flag field*.
This requires that each *solid* cell considered by the boundary condition is marked with a *boundary flag*,
and each valid *fluid* cell is marked with the *fluid* flag.
Assuming that this information is stored in a flag field of type `FlagField_T` identified by `flagFieldId`,
and `boundaryFlagUID` and `fluidFlagUID` respectively identify the boundary and fluid flags,
we can create a sparse boundary sweep using the `fromFlagField` method of the factory:

```{literalinclude} ../examples/FreeSlipBubble/FreeSlipBubble.cpp
:language: C++
:start-after: "// begin freeSlipFromFlagField"
:end-before: "// end freeSlipFromFlagField"
:dedent: 8
```
