(guide-sparse-sweeps)=
# Sparse Sweeps

:::{admonition} Example Code
The code shown in this guide is taken from the *SparseSpiral* example:
{download}`SparseSpiral.zip </zipped-examples/SparseSpiral.zip>`
:::


`sfg-walberla` supports the generation of sparse sweeps, which operate only on a subset of cells
defined by a user-provided *index list*.
This can be useful if a numerical rule needs to be applied only in specific regions of the simulation
domain, especially if those regions have an irregular geometry.

To turn your sweep into a sparse sweep, simply set its `sparse` property to `True`.
To illustrate, here's a sparse sweep that stores the coordinates of each cell
in a field:

```{literalinclude} ../examples/SparseSpiral/SpiralSweep.py
:caption: SpiralSweep.py
:lines: 10-17
:dedent: 4
```

In your C++ code, you must set up an index list which lists all cells the sweep should operate on.
Due to waLBerla's block-structured nature, the index list will actually be a collection of cell vectors,
with one vector per block.
WaLBerla provides the `SparseIndexList` class for this.
The following code sample creates and populates an index list, using a predicate here called `isOnSpiral`.
The index list's internal block data ID must then be passed to the sweep's constructor.

```{literalinclude} ../examples/SparseSpiral/SparseSpiral.cpp
:caption: SparseSpiral.cpp
:lines: 46,56-72
:dedent: 8
:language: C++
```
