# Python Environment for Code Generation

The waLBerla build system will set up a [virtual Python environment][venv] inside its
build tree, and use its Python interpreter to run code generation scripts.
On this page, you can find reference information on how this Python environment can be customized.

## Setting the Base Interpreter

WaLBerla uses [FindPython][FindPython] to locate the base Python interpreter
which will be used to create the virtual environment.
Refer to its documentation for ways to affect the discovery process.

## Adding Additional Packages

To install additional packages into the code generation environment,
first register them in your CMake file using the `walberla_codegen_venv_require` function.
This function can be invoked multiple times to add multiple requirements.
The arguments to `walberla_codegen_venv_require` will be directly forwarded to `pip install`,
so you can include any options `pip install` understands to affect the installation.

Calls to `walberla_codegen_venv_require` will only collect the set of requirements.
To perform the installation, `walberla_codegen_venv_populate` must be called after all
requirements are declared.

:::{card} Example

```CMake
#   First, list requirements
walberla_codegen_venv_require( pycowsay )  # Require a single package
walberla_codegen_venv_require( -r my-requirements.txt )  # Specify a requirements file

#   Then, populate the virtual environment
walberla_codegen_venv_populate()
```

:::

:::{error}
It is an error for your CMake system to call
`walberla_codegen_venv_require` after `walberla_codegen_venv_populate`,
or to call `walberla_codegen_venv_populate` more than once.
:::

[venv]: https://docs.python.org/3/library/venv.html
[FindPython]: https://cmake.org/cmake/help/latest/module/FindPython.html
